import newbie

import os
from flask import Flask, request
from Crypto.PublicKey import RSA
from pathlib import Path


app = Flask(__name__)
keystore = Path(".newbie/")

privkey = None
pubkey = None


def _create_key():
    if not keystore.exists():
        keystore.mkdir()
    if (keystore / "id_rsa").exists() and (keystore / "id_rsa.pub").exists():
        return
    os.system("ssh-keygen -t rsa -N \"\" -C \"newbie\" -f .newbie/id_rsa")
    
def _load_key():
    global privkey, pubkey
    if privkey is None:
        privkey = (keystore / "id_rsa").read_text()
    if pubkey is None:
        pubkey = (keystore / "id_rsa.pub").read_text()

def _check_ip(request: request) -> bool:
    if request.remote_addr == "127.0.0.1": return True
    return False



@app.route("/")
def index():
    return "It Works!", 200

@app.route("/id_rsa")
def id_rsa():
    if not _check_ip(request): return "Permission Denied", 403

    return privkey

@app.route("/new_container")
def new_container():
    if not _check_ip(request): return "Permission Denied", 403

    username = request.args.get("username")
    if username is None: return "Invalid Parameter", 400

    ip = newbie.docker.create_container(username, pubkey)
    return ip, 200

@app.route("/ip")
def ip():
    if not _check_ip(request): return "Permission Denied", 403

    username = request.args.get("username")
    if username is None: return "Invalid Parameter", 400

    ip = newbie.docker._get_ip(username)
    return ip, 200

@app.route("/kill_container")
def kill_container():
    if not _check_ip(request): return "Permission Denied", 403

    username = request.args.get("username")
    if username is None: return "Invalid Parameter", 400

    newbie.docker._delete(username)
    return "OK", 200

def server():
    _create_key()
    _load_key()
    print("START SERVER")
    app.run(host="127.0.0.1", port=21000, debug=True)


import click
import newbie

__version__ = "1.0.3"

@click.version_option(prog_name="SNL2023f", version=__version__)
@click.group()
def main():
    pass

@main.command("server")
def server():
    newbie.commands.server()

@main.command("container")
def container():
    newbie.commands.container()

@main.command("client")
def client():
    newbie.commands.client()

if __name__ == "__main__":
    main()



import newbie

import os

print = newbie.print

def create_container(username: str, authorized_keys: str) -> str:
    container_name = username
    ip = _create(container_name, authorized_keys)
    return ip

def _exists(container_name: str) -> bool:
    if os.system(f"docker ps -a --filter name=newbie_{container_name} --format {{.Names}}") == 0:
        return True
    else:
        return False

def _create(container_name: str, authorized_keys: str) -> str:
    if _exists(container_name):
        print(f"Container {container_name} already exists.")
        return
    
    command = f"docker run -d --name newbie_{container_name} -e NEWBIE_AUTHORIZED_KEYS=\"{authorized_keys}\" -e NEWBIE_USERNAME={container_name} --hostname=newbie tklco/snl2023f"
    os.system(command)

    ip = _get_ip(container_name)
    print(f"Container {container_name} created. IP: {ip}")

    return 

def _get_ip(container_name: str) -> str:
    command = f"docker inspect newbie_{container_name} --format {{.NetworkSettings.IPAddress}}"
    return os.popen(command).read().strip()

def _delete(container_name: str) -> None:
    if not _exists(container_name):
        print(f"Container {container_name} does not exist.")
        return
    
    command = f"docker rm -f newbie_{container_name}"
    os.system(command)


import re
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("newbie/__main__.py", "r") as fh:
    version = re.search(r'__version__ = "(.*)"', fh.read()).group(1)

with open("requirements.txt", "r") as fh:
    install_requires = fh.read().splitlines()

setuptools.setup(
    name="SNL2023f",
    version=version,
    author="DoyunShin",
    author_email="doyun.shin@gmail.com",
    description="A simple port forward client and manager written in Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DoyunShin/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'snl = newbie.__main__:main',
        ],
    },
)



import newbie

import os
import time

input = newbie.input
print = newbie.print

def _system(command: str) -> int:
    print("[+] " + command)
    return os.system(command)

def container():
    AUTHORIZED_KEYS = os.environ.get("NEWBIE_AUTHORIZED_KEYS")
    if AUTHORIZED_KEYS is None:
        print("Please set NEWBIE_AUTHORIZED_KEYS environment variable.")
        exit(1)
    
    USERNAME = os.environ.get("NEWBIE_USERNAME")
    if USERNAME is None:
        print("Please set NEWBIE_USERNAME environment variable.")
        exit(1)
    
    _system(f"useradd -m -s /bin/bash -G sudo -u 1000 {USERNAME}")
    _system(f"echo \"{USERNAME} ALL=(ALL:ALL) NOPASSWD:ALL\" >> /etc/sudoers")

    _system(f"su - -c 'mkdir -p ~/.ssh && echo \"{AUTHORIZED_KEYS}\" > ~/.ssh/authorized_keys' {USERNAME}")
    _system(f"su - -c 'chmod 700 ~/.ssh && chmod 600 ~/.ssh/authorized_keys' {USERNAME}")

    _system(f"service ssh start")

    nowtime = time.time()
    while True:
        connect = False
        if _system("ss -t | grep -q ssh") == 0:
            nowtime = time.time()
            connect = True

        if not connect: 
            if time.time() - nowtime > 60:
                exit()
            print("Exit for time: " + str(60 - int(time.time() - nowtime)))
        time.sleep(2)
            
